SystemOrganization addCategory: #'Pier-Security-Command'!
SystemOrganization addCategory: #'Pier-Security-Model'!
SystemOrganization addCategory: #'Pier-Security-Widgets'!

PRChangeCommand subclass: #PUEditUser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Security-Command'!

!PUEditUser class methodsFor: 'testing' stamp: 'NickAger 2/28/2010 05:11'!
isAbstract
	^ false! !

!PUEditUser class methodsFor: 'testing' stamp: 'NickAger 2/28/2010 05:11'!
isValidIn: aContext
	^ aContext isLoggedIn! !

!PUEditUser class methodsFor: 'accessing' stamp: 'NickAger 2/28/2010 05:12'!
label
	^ 'Edit User'! !

!PUEditUser class methodsFor: 'accessing' stamp: 'lr 2/28/2010 17:42'!
permissions
	^ #()! !

!PUEditUser methodsFor: 'testing' stamp: 'NickAger 2/28/2010 10:50'!
description
	^ self context user description
		select: [ :each | each hasProperty: #public ]
! !

!PUEditUser methodsFor: 'events' stamp: 'lr 2/28/2010 18:25'!
doValidate
	"Change user commands do not require validation."! !

!PUEditUser methodsFor: 'accessing' stamp: 'lr 2/28/2010 17:52'!
target
	^ self context user! !

PRDecoration subclass: #PUSecurity
	instanceVariableNames: 'owner ownerPermissions group groupPermissions otherPermissions'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Security-Model'!

!PUSecurity commentStamp: 'lr 12/20/2005 15:05' prior: 0!
Every structure is owned by a specific user. The owner of an object comprises its user class. Permissions assigned to the user class only apply to that specific user. A structure is also assigned a group, which comprises its group class. Permissions assigned to the group class only apply to members of that group. Users who are not otherwise represented by the other two classes comprise a file's others class.

The effective permissions that have applied to a specific user in relation to a file are determined in logical precedence. For example, the user who owns the file will have the effective permissions given to the user class regardless of those assigned to the group or others class.!

!PUSecurity class methodsFor: 'testing' stamp: 'lr 3/9/2006 11:31'!
isAbstract
	^ false! !

!PUSecurity class methodsFor: 'events' stamp: 'lr 6/17/2010 09:43'!
onStructureAdded: aStructure 
	| decoration context user |
	decoration := aStructure isRoot 
		ifTrue: [ aStructure securityDecorationClass new ]
		ifFalse: [ aStructure parent securityDecoration copy ].
	(context := PRCurrentContext value) isNil ifFalse: [
		(user := context user) isNil ifFalse: [
			decoration owner: user.
			decoration group ifNil: [ decoration group: user group ] ] ].
	^ aStructure 
		addDecoration: decoration
		ifPresent: [ :value | value ]! !

!PUSecurity methodsFor: 'visiting' stamp: 'lr 3/9/2006 11:31'!
accept: aVisitor
	aVisitor visitSecurity: self! !

!PUSecurity methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:31'!
group
	^ group! !

!PUSecurity methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:31'!
group: aGroup
	group := aGroup! !

!PUSecurity methodsFor: 'accessing-permissions' stamp: 'lr 9/11/2006 10:28'!
groupPermissions
	^ groupPermissions ifNil: [ groupPermissions := self ownerPermissions copy ]! !

!PUSecurity methodsFor: 'accessing-permissions' stamp: 'lr 9/11/2006 10:28'!
groupPermissions: aSet
	groupPermissions := aSet asSet! !

!PUSecurity methodsFor: 'testing' stamp: 'lr 4/29/2009 23:29'!
hasPermission: aPermission for: aUser
	"Test if the user aUser has the permission aPermission. This is the central method to test for permissions in Pier."

	(aUser notNil and: [ aUser isSuperuser ])
		ifTrue: [ ^ true ].
	(self owner = aUser and: [ self ownerPermissions includes: aPermission ])
		ifTrue: [ ^ true ].
	(self group notNil and: [ (self group includes: aUser) and: [ (self groupPermissions includes: aPermission) ] ])
		ifTrue: [ ^ true ].
	(self otherPermissions includes: aPermission)
		ifTrue: [ ^ true ].
	^ false! !

!PUSecurity methodsFor: 'initialization' stamp: 'lr 9/11/2006 10:48'!
initialize
	super initialize.
	self ownerPermissions; groupPermissions; otherPermissions! !

!PUSecurity methodsFor: 'testing' stamp: 'lr 4/29/2009 23:30'!
isAllowedCommand: aCommandClass in: aContext
	"Answer if aCommandClass has any permissions in the current context."

	^ aCommandClass permissions
		anySatisfy: [ :permission | self hasPermission: permission for: aContext user ]! !

!PUSecurity methodsFor: 'accessing-permissions' stamp: 'lr 9/11/2006 10:29'!
otherPermissions
	^ otherPermissions ifNil: [ otherPermissions := Set withAll: PRViewCommand permissions ]! !

!PUSecurity methodsFor: 'accessing-permissions' stamp: 'lr 9/11/2006 10:29'!
otherPermissions: aSet
	otherPermissions := aSet asSet! !

!PUSecurity methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:31'!
owner
	^ owner! !

!PUSecurity methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:31'!
owner: anUser
	owner := anUser! !

!PUSecurity methodsFor: 'accessing-permissions' stamp: 'lr 9/11/2006 10:29'!
ownerPermissions
	^ ownerPermissions ifNil: [ ownerPermissions := PUChangeSecurity permissionInstances asSet ]! !

!PUSecurity methodsFor: 'accessing-permissions' stamp: 'lr 9/11/2006 10:30'!
ownerPermissions: aSet
	ownerPermissions := aSet asSet! !

!PUSecurity methodsFor: 'copying' stamp: 'lr 9/11/2006 10:30'!
postCopy
	super postCopy.
	self ownerPermissions: self ownerPermissions copy.
	self groupPermissions: self groupPermissions copy.
	self otherPermissions: self otherPermissions copy! !

!PUSecurity methodsFor: 'configuration' stamp: 'lr 3/9/2006 11:31'!
priority
	^ -200! !

!PUSecurity methodsFor: 'testing' stamp: 'lr 4/29/2009 23:30'!
validateCommand: aCommand
	"Validate the evaluation of aCommand on the owning structure. Throws an error if this violates ther current permissions."

	(self hasPermission: aCommand permission for: aCommand context user)
		ifFalse: [ MAConditionError description: self description signal: 'Insufficient permissions' ]! !

!GRPackage class methodsFor: '*pier-security' stamp: 'lr 2/18/2010 10:35'!
pierSecurity
	^ self new
		name: 'Pier-Security';
		description: 'A unix-like security system for Pier.';
		addDependency: 'Pier-Model';
		addDependency: 'Pier-Seaside';
		addDependency: 'Seaside-Widgets';
		url: #pierUrl;
		yourself! !

!PRContext class methodsFor: '*pier-security' stamp: 'lr 5/5/2009 16:58'!
descriptionUser
	^ MASingleOptionDescription new
		accessor: (MASelectorAccessor read: #user write: #setUser:);
		reference: PUUser description;
		parameterName: 'user';
		priority: 500;
		label: 'User';
		yourself! !

!PRContext methodsFor: '*pier-security-accessing' stamp: 'lr 3/9/2006 11:30'!
group
	^ self user group! !

!PRContext methodsFor: '*pier-security-testing' stamp: 'lr 3/9/2006 11:30'!
isLoggedIn
	^ self user notNil! !

!PRContext methodsFor: '*pier-security-private' stamp: 'lr 4/23/2011 08:32'!
rememberUserInCookie: aUser
	| cookie context application serverPath |
	context := self requestContextIfNilDo: [ ^ nil ].
	application := context application.
	
	serverPath := application serverPath.
	serverPath isNil ifTrue: [
		| dispatcher |
		dispatcher := application parent.
		"check if the current application is set as the default dispatcher"
		serverPath := (dispatcher notNil and: [ dispatcher isDispatcher and: [ dispatcher defaultHandler = application ] ]) 
			ifTrue: [ '/' ]
			ifFalse: [ application url pathString ] ].
	
	cookie := WACookie new key: 'pr_token'; path: serverPath; yourself.
	aUser isNil
		ifTrue: [ cookie value: ''; expireIn: -356 days ]
		ifFalse: [ cookie value: aUser token; expireIn: 7 days ].
	context response addCookie: cookie! !

!PRContext methodsFor: '*pier-security-private' stamp: 'jf 9/14/2009 16:30'!
retrieveUserFromCookie
	| cookie |
	cookie := ((self requestContextIfNilDo: [ ^ nil ]) request
		cookieAt: 'pr_token')
			ifNil: [ ^ nil ].
	^ self kernel users
		detect: [ :each | each token = cookie value ]
		ifNone: [ nil ]! !

!PRContext methodsFor: '*pier-security-initialization' stamp: 'lr 4/28/2009 22:15'!
setUser: aUser
	self propertyAt: #user put: aUser.
	self rememberUserInCookie: aUser! !

!PRContext methodsFor: '*pier-security-accessing' stamp: 'lr 4/30/2009 15:18'!
user
	^ self propertyAt: #user ifAbsentPut: [ self retrieveUserFromCookie ]! !

!PRContext methodsFor: '*pier-security-navigation' stamp: 'lr 3/9/2006 11:30'!
user: anUser
	"Create a copy of the current context with the current user replaced by anUser."

	^ self copy
		setUser: anUser;
		yourself! !

!PRKernel methodsFor: '*pier-security-defaults' stamp: 'lr 9/27/2006 11:45'!
adminGroup
	^ self propertyAt: #adminGroup ifAbsentPut: [ PUGroup named: 'admin' ]! !

!PRKernel methodsFor: '*pier-security-defaults' stamp: 'lr 9/27/2006 11:45'!
adminUser
	^ self propertyAt: #adminUser ifAbsentPut: [
		(PUUser named: 'admin')
			addGroup: self adminGroup;
			password: 'pier';
			superuser: true;
			yourself ]! !

!PRKernel methodsFor: '*pier-security-accessing' stamp: 'lr 2/5/2009 21:34'!
groupNamed: aString ifNone: aBlock
	^ self groups 
		detect: [ :each | each name = aString ]
		ifNone: aBlock! !

!PRKernel methodsFor: '*pier-security-accessing' stamp: 'lr 3/9/2006 11:30'!
groups
	^ self propertyAt: #groups ifAbsentPut: [ Set with: self adminGroup ]! !

!PRKernel methodsFor: '*pier-security-accessing' stamp: 'lr 2/5/2009 21:58'!
userNamed: aString ifNone: aBlock
	^ self users
		detect: [ :each | each name = aString ]
		ifNone: aBlock! !

!PRKernel methodsFor: '*pier-security-accessing' stamp: 'lr 3/9/2006 11:30'!
users
	^ self propertyAt: #users ifAbsentPut: [ Set with: self adminUser ]! !

!PRStructure class methodsFor: '*pier-security-descriptions' stamp: 'lr 3/26/2009 20:58'!
descriptionGroup
	^ MAToOneRelationDescription new
		classes: (Array with: PUGroup);
		parameterName: 'group';
		accessor: #group;
		label: 'Group';
		priority: 210;
		yourself! !

!PRStructure class methodsFor: '*pier-security-descriptions' stamp: 'lr 3/26/2009 20:58'!
descriptionOwner
	^ MAToOneRelationDescription new
		classes: (Array with: PUUser);
		parameterName: 'owner';
		accessor: #owner;
		label: 'Owner';
		priority: 200;
		yourself! !

!PRStructure methodsFor: '*pier-security-accessing' stamp: 'lr 6/17/2010 09:42'!
group
	"Answer the owning-group of the file."

	| decoration |
	^ (decoration := self securityDecoration) isNil ifFalse: [ decoration group ]! !

!PRStructure methodsFor: '*pier-security-accessing' stamp: 'lr 6/17/2010 09:42'!
owner
	"Answer the owning-user of the receiver."

	| decoration |
	^ (decoration := self securityDecoration) isNil ifFalse: [ decoration owner ]! !

!PRStructure methodsFor: '*pier-security-decorations' stamp: 'lr 3/9/2006 11:30'!
securityDecoration
	^ self
		decorationOfClass: self securityDecorationClass
		ifAbsent: [ self securityDecorationClass onStructureAdded: self ]! !

!PRStructure methodsFor: '*pier-security-decorations' stamp: 'lr 3/9/2006 11:30'!
securityDecorationClass
	^ PUSecurity! !

!PRVisitor methodsFor: '*pier-security-visiting-structure' stamp: 'lr 10/16/2007 22:28'!
visitSecurity: anObject! !

!PRValueLink methodsFor: '*pier-security' stamp: 'lr 4/1/2009 21:46'!
groupIn: aContext
	<value: 'group' comment: 'Display the group of the current structure.'>
	
	^ self lookupStructure: aContext structure do: [ :structure |
		self formatDescribed: structure group default: #name ]! !

!PRValueLink methodsFor: '*pier-security' stamp: 'lr 4/1/2009 21:46'!
ownerIn: aContext
	<value: 'owner' comment: 'Display the owner of the current structure.'>

	^ self lookupStructure: aContext structure do: [ :structure |
		self formatDescribed: structure owner default: #name ]! !

!PRValueLink methodsFor: '*pier-security' stamp: 'lr 4/1/2009 21:46'!
userIn: aContext
	<value: 'user' comment: 'Display the currently authenticated structure.'>

	^ self formatDescribed: aContext user default: #name! !

MAValidationError subclass: #PUInvalidLogin
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Security-Model'!

!PRAddCommand class methodsFor: '*pier-security-private' stamp: 'jf 9/17/2009 15:43'!
descriptionTypeSecurity: aDescription
	"Only list the structures with that the current user can actually add."

	^ aDescription
		options: [ self structureClassesSecurity ] magritteDynamicObject;
		yourself! !

!PRAddCommand class methodsFor: '*pier-security' stamp: 'lr 4/30/2009 00:04'!
permissions
	^ PRStructure withAllConcreteClasses
		collect: [ :class | PUAddPermission on: self structure: class ]! !

!PRAddCommand class methodsFor: '*pier-security-private' stamp: 'lr 4/30/2009 00:03'!
structureClassesSecurity
	| context |
	context := PRCurrentContext value.
	^ self structureClasses select: [ :class |
		context structure securityDecoration
			hasPermission: (PUAddPermission
				on: self structure: class)
			for: context user ]! !

!PRAddCommand methodsFor: '*pier-security' stamp: 'lr 4/30/2009 00:03'!
permission
	^ PUAddPermission on: self class structure: self type! !

PRViewComponent subclass: #PUBrowseComponent
	instanceVariableNames: 'permissions structure'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Security-Widgets'!

!PUBrowseComponent class methodsFor: 'testing' stamp: 'lr 10/17/2006 07:15'!
isAbstract
	^ false! !

!PUBrowseComponent class methodsFor: 'accessing' stamp: 'lr 10/17/2006 07:18'!
label
	^ 'Browse'! !

!PUBrowseComponent methodsFor: 'configuration' stamp: 'lr 2/12/2007 19:07'!
defaultPermissions
	^ (Array streamContents: [ :stream |
		PRCommand allSubclasses do: [ :each |
			each isAbstract
				ifFalse: [ stream nextPutAll: each permissions ] ] ])
		asSortedCollection: [ :a :b | a label < b label ]! !

!PUBrowseComponent methodsFor: 'initialization' stamp: 'lr 1/8/2010 22:23'!
initialize
	super initialize.
	permissions := self defaultPermissions! !

!PUBrowseComponent methodsFor: 'accessing' stamp: 'lr 10/17/2006 07:20'!
permissions
	^ permissions! !

!PUBrowseComponent methodsFor: 'rendering' stamp: 'lr 9/7/2009 20:46'!
renderContentOn: html
	self renderPathOn: html.
	self renderTableOn: html! !

!PUBrowseComponent methodsFor: 'rendering' stamp: 'lr 1/8/2010 22:31'!
renderPathOn: html
	html div: [
		self structure parents do: [ :each |
			html text: '/ '.
			html anchor
				callback: [ self structure: each ];
				title: each title;
				with: each name.
			html text: ' ' ] ].
	html break! !

!PUBrowseComponent methodsFor: 'rendering' stamp: 'lr 10/25/2010 08:52'!
renderPermissions: aStructure permissions: aCollection on: html
	| current short |
	current := self context structure: aStructure.
	self permissions do: [ :each |
		short := (aCollection includes: each)
			ifTrue: [ each label first asLowercase ]
			ifFalse: [ $- ].
		(each command isValidIn: current)
			ifFalse: [ 
				html span 
					title: each label; 
					with: short ]
			ifTrue: [
				html anchor
					goto: (self context
						structure: aStructure
						command: (each command new
							successAnswer: self context;
							cancelAnswer: self context;
							yourself));
					title: each label;
					with: short ] ]! !

!PUBrowseComponent methodsFor: 'rendering' stamp: 'lr 3/18/2008 21:49'!
renderTableHeadOn: html
	html tableRow: [
		html tableHeading: 'name'.
		html tableHeading: 'type'.
		html tableHeading: 'owner mode'.
		html tableHeading: 'group mode'.
		html tableHeading: 'other mode'.
		html tableHeading: 'owner'.
		html tableHeading: 'group' ]! !

!PUBrowseComponent methodsFor: 'rendering' stamp: 'lr 1/8/2010 22:44'!
renderTableOn: html
	html table: [
		html tableHead: [
			self renderTableHeadOn: html ].
		html tableBody: [
			self structure parent isNil
				ifFalse: [ self renderTableRow: self structure parent named: '..' on: html ].
			self renderTableRow: self structure named: '.' on: html.
			self structure enumerator do: [ :each |
				(each isValidCommand: each viewCommandClass in: (self context structure: structure))
					ifTrue: [ self renderTableRow: each named: each name on: html ] ] ] ]! !

!PUBrowseComponent methodsFor: 'rendering' stamp: 'lr 1/8/2010 22:24'!
renderTableRow: aStructure named: aString on: html
	| security |
	security := aStructure securityDecoration.
	html tableRow attributeAt: 'title' put: aStructure title; with: [
		html tableData: [
			html anchor
				callback: [ self structure: aStructure ];
				title: aStructure title;
				with: aString ].
		html tableData: aStructure class label.
		html tableData: [
			self
				renderPermissions: aStructure
				permissions: security ownerPermissions
				on: html ].
		html tableData: [
			self
				renderPermissions: aStructure
				permissions:security groupPermissions
				on: html ].
		html tableData: [
			self
				renderPermissions: aStructure
				permissions: security otherPermissions
				on: html ].
		html tableData: (security owner isNil
			ifFalse: [ security owner name ]).
		html tableData: (security group isNil
			ifFalse: [ security group name ]) ]! !

!PUBrowseComponent methodsFor: 'accessing' stamp: 'lr 1/8/2010 22:23'!
structure
	^ structure ifNil: [ structure := self context structure ]! !

!PUBrowseComponent methodsFor: 'accessing' stamp: 'lr 1/8/2010 22:23'!
structure: aStructure
	structure := aStructure! !

PRWidget subclass: #PRFileSettings
	instanceVariableNames: 'component'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Security-Widgets'!

!PRFileSettings class methodsFor: 'accessing' stamp: 'lr 2/13/2009 20:43'!
groupLabel
	^ 'Pier Settings'! !

!PRFileSettings class methodsFor: 'testing' stamp: 'lr 2/13/2009 20:33'!
isAbstract
	^ false! !

!PRFileSettings class methodsFor: 'accessing' stamp: 'lr 2/13/2009 20:33'!
label
	^ 'File Settings'! !

!PRFileSettings methodsFor: 'accessing' stamp: 'lr 2/13/2009 20:40'!
children
	^ Array with: component! !

!PRFileSettings methodsFor: 'accessing' stamp: 'jf 9/28/2009 13:34'!
description
	^ MAContainer new
		add: (MAStringDescription new
			label: 'Base Directory';
			accessor: #baseDirectoryPath;
			comment: 'Defines the base-directory where the files are stored. If this field is left empty, it default to a subdirectory of of the current image-location.';
			yourself);
		add: (MAStringDescription new
			label: 'Base URL';
			accessor: #baseUrl;
			comment: 'Defines the base-URL where the files are served from, when using an external web server. if this field is left empty, files are served through the image.';
			yourself);
		yourself! !

!PRFileSettings methodsFor: 'initialization' stamp: 'lr 2/13/2009 20:57'!
initialize
	super initialize.
	component := (self description asComponentOn: self)
		addValidatedForm;
		yourself! !

!PRFileSettings methodsFor: 'accessing' stamp: 'jf 9/28/2009 11:17'!
model
	^ PRFile fileModel! !

!PRFileSettings methodsFor: 'dispatching' stamp: 'jf 9/28/2009 13:34'!
readUsing: aDescription
	^ self model readUsing: aDescription
		! !

!PRFileSettings methodsFor: 'rendering' stamp: 'lr 7/26/2010 16:35'!
renderContentOn: html
	self isSuperuser
		ifFalse: [ ^ html paragraph: 'Only adminstrators can edit the file settings.' ].
	html render: self children! !

!PRFileSettings methodsFor: 'dispatching' stamp: 'lr 2/13/2009 21:02'!
write: anObject using: aDescription
	^ self model write: anObject using: aDescription! !

PRWidget subclass: #PRKernelSettings
	instanceVariableNames: 'component'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Security-Widgets'!

!PRKernelSettings class methodsFor: 'accessing' stamp: 'lr 2/13/2009 20:43'!
groupLabel
	^ 'Pier Settings'! !

!PRKernelSettings class methodsFor: 'testing' stamp: 'lr 9/3/2008 13:55'!
isAbstract
	^ false! !

!PRKernelSettings class methodsFor: 'accessing' stamp: 'lr 2/13/2009 20:24'!
label
	^ 'Kernel Settings'! !

!PRKernelSettings methodsFor: 'accessing' stamp: 'lr 9/3/2008 13:53'!
children
	^ Array with: component! !

!PRKernelSettings methodsFor: 'initialization' stamp: 'lr 9/3/2008 13:51'!
initialize
	super initialize.
	component := self kernel asComponent
		addValidatedForm;
		yourself! !

!PRKernelSettings methodsFor: 'rendering' stamp: 'lr 7/26/2010 16:35'!
renderContentOn: html
	self isSuperuser
		ifFalse: [ ^ html paragraph: 'Only adminstrators can edit the kernel settings.' ].
	html render: self children! !

PRWidget subclass: #PRPersistencySettings
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Security-Widgets'!

!PRPersistencySettings class methodsFor: 'accessing' stamp: 'lr 2/13/2009 20:43'!
groupLabel
	^ 'Pier Settings'! !

!PRPersistencySettings class methodsFor: 'testing' stamp: 'lr 9/3/2008 11:59'!
isAbstract
	^ false! !

!PRPersistencySettings class methodsFor: 'accessing' stamp: 'lr 2/13/2009 20:25'!
label
	^ 'Persistency Settings'! !

!PRPersistencySettings methodsFor: 'actions' stamp: 'lr 9/3/2008 12:08'!
activate: aClass
	self kernel persistency class = aClass
		ifFalse: [ self kernel persistency: aClass new ]! !

!PRPersistencySettings methodsFor: 'actions' stamp: 'lr 9/4/2009 17:31'!
edit: aPersistency
	self show: (aPersistency asComponent
		addValidatedForm;
		yourself)! !

!PRPersistencySettings methodsFor: 'rendering' stamp: 'lr 7/26/2010 16:35'!
renderContentOn: html
	self isSuperuser
		ifFalse: [ ^ html paragraph: 'Only adminstrators can edit the persistency settings.' ].
	html form: [
		html select
			labels: [ :each | each label ];
			selected: self kernel persistency class;
			list: PRPersistency withAllConcreteClasses;
			callback: [ :each | self activate: each ].
		html space.
		html submitButton
			with: 'select'.
		5 timesRepeat: [ html space ].
		html submitButton
			callback: [ self edit: self kernel persistency ];
			disabled: self kernel persistency description isEmpty;
			with: 'edit'.
		(self kernel persistency respondsTo: #snapshot) ifTrue: [
			html space.
			html submitButton
				callback: [ self kernel persistency snapshot ];
				text: 'snapshot' ] ].
	html paragraph: self kernel persistency class comment! !

!PRWidget methodsFor: '*pier-security-testing' stamp: 'lr 9/27/2006 14:19'!
isSuperuser
	^ self context user notNil and: [ self context user isSuperuser ]! !

PRWidget subclass: #PUMemberWidget
	instanceVariableNames: 'report'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Security-Widgets'!

PUMemberWidget subclass: #PUGroupsWidget
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Security-Widgets'!

!PUGroupsWidget class methodsFor: 'testing' stamp: 'lr 9/27/2006 10:30'!
isAbstract
	^ false! !

!PUGroupsWidget class methodsFor: 'accessing' stamp: 'lr 9/3/2008 11:50'!
label
	^ 'Groups'! !

!PUGroupsWidget methodsFor: 'accessing' stamp: 'lr 9/27/2006 10:52'!
memberClass
	^ PUGroup! !

!PUGroupsWidget methodsFor: 'accessing' stamp: 'lr 9/27/2006 10:54'!
members
	^ self context kernel groups! !

!PUGroupsWidget methodsFor: 'actions' stamp: 'lr 9/4/2009 17:42'!
remove: aMember
	aMember users isEmpty ifFalse: [ 
		^ self show: (WAFormDialog new
			addMessage: 'Remove all users before removing a group.';
			yourself) ].
	super remove: aMember! !

!PUMemberWidget class methodsFor: 'accessing' stamp: 'lr 9/3/2008 11:46'!
groupLabel
	^ 'Pier System'! !

!PUMemberWidget methodsFor: 'actions' stamp: 'lr 9/4/2009 17:40'!
add: aString
	aString isEmptyOrNil
		ifTrue: [ ^ self ].
	self members do: [ :each |
		each name = aString ifTrue: [ 
			self show: (WAFormDialog new
				addMessage: 'A member with the name ' , aString printString , ' already exists.';
				yourself) ] ].
	self
		edit: (self memberClass named: aString) title: 'Add'
		do: [ :member | 
			self members add: member.
			self refresh	]! !

!PUMemberWidget methodsFor: 'private' stamp: 'lr 4/23/2009 22:38'!
buildReport
	| component |
	component := MAReport
		rows: self members asArray
		description: (self memberClass description
			select: [ :each | each hasProperty: #report ]).
	component addColumn: (MACommandColumn new
		visible: self isSuperuser;
		addCommandOn: self selector: #edit: text: 'edit';
		addCommandOn: self selector: #remove: text: 'remove';
		yourself).
	^ component! !

!PUMemberWidget methodsFor: 'accessing' stamp: 'lr 2/12/2007 19:07'!
children
	^ Array with: self report! !

!PUMemberWidget methodsFor: 'actions' stamp: 'lr 9/4/2009 17:39'!
edit: aMember
	self edit: aMember title: 'Edit' do: [ :member | self refresh ]! !

!PUMemberWidget methodsFor: 'actions' stamp: 'lr 9/4/2009 17:37'!
edit: aMember title: aString do: aBlock
	self isSuperuser 
		ifFalse: [ ^ nil ].
	self 
		show: (aMember asComponent
			addMessage: aString;
			addValidatedForm;
			yourself)
		onAnswer: [ :answer | 
			answer notNil
				ifTrue: [ aBlock value: answer ] ]! !

!PUMemberWidget methodsFor: 'accessing' stamp: 'lr 10/30/2006 19:30'!
memberClass
	self subclassResponsibility! !

!PUMemberWidget methodsFor: 'accessing' stamp: 'lr 10/30/2006 19:29'!
members
	self subclassResponsibility! !

!PUMemberWidget methodsFor: 'events' stamp: 'lr 9/26/2007 19:15'!
onChangeContext: aContext
	super onChangeContext: aContext.
	self report columns last visible: self isSuperuser! !

!PUMemberWidget methodsFor: 'actions' stamp: 'lr 9/27/2006 11:01'!
refresh
	self report rows: self members asArray! !

!PUMemberWidget methodsFor: 'actions' stamp: 'lr 9/4/2009 17:45'!
remove: aMember
	self remove: aMember do: [ :answer | ]! !

!PUMemberWidget methodsFor: 'actions' stamp: 'lr 9/4/2009 17:46'!
remove: aMember do: aBlock
	self isSuperuser 
		ifFalse: [ ^ nil ].
	self
		show: (WAYesOrNoDialog new
			addMessage: 'Are you sure to remove ' , aMember name printString , '?';
			yourself)
		onAnswer: [ :answer |
			answer ifTrue: [
				self members remove: aMember.
				self refresh.
				aBlock value: aMember ] ]! !

!PUMemberWidget methodsFor: 'rendering' stamp: 'lr 9/27/2006 12:12'!
renderAddOn: html
	self isSuperuser
		ifFalse: [ ^ self ].
	html break.
	html form: [
		html textInput
			callback: [ :value | self add: value ].
		html submitButton
			text: 'add' ]! !

!PUMemberWidget methodsFor: 'rendering' stamp: 'lr 4/23/2007 07:43'!
renderContentOn: html
	self renderReportOn: html.
	self renderAddOn: html! !

!PUMemberWidget methodsFor: 'rendering' stamp: 'lr 9/27/2006 10:59'!
renderReportOn: html
	html render: self report! !

!PUMemberWidget methodsFor: 'accessing' stamp: 'lr 9/27/2006 10:58'!
report
	^ report ifNil: [ report := self buildReport ]! !

PUMemberWidget subclass: #PUUsersWidget
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Security-Widgets'!

!PUUsersWidget class methodsFor: 'testing' stamp: 'lr 9/27/2006 10:30'!
isAbstract
	^ false! !

!PUUsersWidget class methodsFor: 'accessing' stamp: 'lr 9/3/2008 11:50'!
label
	^ 'Users'! !

!PUUsersWidget methodsFor: 'accessing' stamp: 'lr 9/27/2006 10:52'!
memberClass
	^ PUUser! !

!PUUsersWidget methodsFor: 'accessing' stamp: 'lr 9/27/2006 10:54'!
members
	^ self context kernel users! !

!PUUsersWidget methodsFor: 'actions' stamp: 'lr 9/4/2009 17:48'!
remove: aMember
	self context user = aMember ifTrue: [ 
		^ self show: (WAFormDialog new
			addMessage: 'You cannot remove yourself.';
			yourself) ].
	self remove: aMember do: [ :member | member groups: #() ]! !

PRWidget subclass: #PUReportWidget
	instanceVariableNames: 'report'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Security-Widgets'!

!PUReportWidget class methodsFor: 'accessing' stamp: 'lr 4/22/2009 17:05'!
groupLabel
	^ 'Pier System'! !

!PUReportWidget methodsFor: 'private' stamp: 'lr 4/22/2009 17:04'!
buildReport
	^ (MAReport rows: self rows)
		showFilter: true;
		batchSize: 25;
		yourself! !

!PUReportWidget methodsFor: 'accessing' stamp: 'lr 4/22/2009 16:58'!
children
	^ Array with: self report! !

!PUReportWidget methodsFor: 'actions' stamp: 'jf 9/6/2009 20:53'!
export
	self requestContext response
		document: self report export
			mimeType: 'application/excel'
			fileName: self context structure name , '.xls';
		respond! !

!PUReportWidget methodsFor: 'actions' stamp: 'lr 4/22/2009 17:01'!
refresh
	self report rows: self rows! !

!PUReportWidget methodsFor: 'rendering' stamp: 'lr 5/28/2009 11:39'!
renderActionsOn: html 
	html form: [ self renderButtonsOn: html ]! !

!PUReportWidget methodsFor: 'rendering' stamp: 'lr 5/28/2009 11:39'!
renderButtonsOn: html
	html submitButton
		callback: [ self refresh ];
		with: 'Refresh'.
	html submitButton
		callback: [ self export ];
		with: 'Export'! !

!PUReportWidget methodsFor: 'rendering' stamp: 'lr 4/22/2009 16:59'!
renderContentOn: html
	self renderReportOn: html.
	self renderActionsOn: html! !

!PUReportWidget methodsFor: 'rendering' stamp: 'lr 4/22/2009 16:59'!
renderReportOn: html
	html render: self report! !

!PUReportWidget methodsFor: 'rendering' stamp: 'lr 4/22/2009 17:16'!
renderStructure: aStructure on: html
	html anchor
		goto: (self context structure: aStructure);
		with: aStructure absolutePath! !

!PUReportWidget methodsFor: 'accessing' stamp: 'lr 4/22/2009 16:58'!
report
	^ report ifNil: [ report := self buildReport ]! !

!PUReportWidget methodsFor: 'accessing' stamp: 'DaleHenrichs 06/25/2010 13:27'!
rows
	self subclassResponsibility! !

PUReportWidget subclass: #PUStructuresWidget
	instanceVariableNames: 'query'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Security-Widgets'!

!PUStructuresWidget class methodsFor: 'testing' stamp: 'lr 4/22/2009 17:05'!
isAbstract
	^ false! !

!PUStructuresWidget class methodsFor: 'accessing' stamp: 'lr 4/22/2009 17:05'!
label
	^ 'Structures'! !

!PUStructuresWidget methodsFor: 'private' stamp: 'lr 4/22/2009 17:17'!
buildReport
	| component |
	component := super buildReport.
	component addColumn: (MAColumn new
		accessor: #title;
		label: 'Title';
		yourself).
	component addColumn: (MAColumn new
		renderer: [ :value :html | self renderStructure: value on: html ];
		formatter: [ :value | value absolutePath ];
		accessor: #yourself;
		label: 'Location';
		yourself).
	component addColumn: (MAColumn new
		accessor: #(class label);
		label: 'Kind';
		filter: true;
		yourself).
	component addColumn: (MAColumn new
		accessor: #(owner name);
		label: 'Owner';
		filter: true;
		yourself).
	component addColumn: (MAColumn new
		accessor: #(group name);
		label: 'Group';
		filter: true;
		yourself).
	^ component! !

!PUStructuresWidget methodsFor: 'accessing' stamp: 'lr 10/5/2010 15:21'!
query: aString 
	| tokens |
	(query := aString) isNil ifTrue: [ ^ self report rowFilter: nil ].
	tokens := aString findTokens: Character space.
	tokens isEmpty ifTrue: [ ^ self report rowFilter: nil ].
	self report rowFilter: [ :row | 
		| values string |
		values := row description children
			collect: [ :description | description toString: (row readUsing: description) ].
		values isEmpty 
			ifTrue: [ false ]
			ifFalse: [ 
				string := values reduce: [ :a :b | a , (String with: Character cr) , b ].
				tokens allSatisfy: [ :token | 
					token first = $- 
						ifTrue: [ (string includesSubstring: token allButFirst caseSensitive: false) not ]
						ifFalse: [ string includesSubstring: token caseSensitive: false ] ] ] ]! !

!PUStructuresWidget methodsFor: 'rendering' stamp: 'lr 4/24/2009 13:49'!
renderContentOn: html
	self renderFilterOn: html.
	super renderContentOn: html! !

!PUStructuresWidget methodsFor: 'rendering' stamp: 'lr 4/24/2009 14:22'!
renderFilterOn: html
	html form class: 'magritte'; with: [
		html table: [ 
			html tableRow: [
				html tableHeading
					title: 'Separate different terms by spaces. To exclude terms prefix with a minus.';
					with: 'Search Query:'.
				html tableData: [
					html textInput
						value: query;
						callback: [ :value | self query: value ].
					html space; submitButton: 'Filter' ] ] ] ]! !

!PUStructuresWidget methodsFor: 'accessing' stamp: 'lr 4/22/2009 17:06'!
rows
	^ (self context enumeratorFor: self context root) 
		everything contents! !

PRWidget subclass: #PUSecurityWidget
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Security-Widgets'!

!PUSecurityWidget class methodsFor: 'accessing' stamp: 'lr 9/3/2008 11:46'!
groupLabel
	^ 'Pier System'! !

!PUSecurityWidget class methodsFor: 'testing' stamp: 'lr 9/27/2006 12:16'!
isAbstract
	^ false! !

!PUSecurityWidget class methodsFor: 'accessing' stamp: 'lr 9/3/2008 11:50'!
label
	^ 'Credentials'! !

!PUSecurityWidget methodsFor: 'rendering' stamp: 'lr 10/15/2010 10:11'!
renderContentOn: html
	| user |
	user := self context user.
	user isNil ifTrue: [ ^ self ].
	html span 
		class: 'changePassword'; 
		with: [
			html anchor
				goto: (self context command: PUEditUser new); 
				with: 'change password' ]! !

PRObject subclass: #PUMember
	instanceVariableNames: 'name comment'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Security-Model'!

PUMember subclass: #PUGroup
	instanceVariableNames: 'users'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Security-Model'!

!PUGroup class methodsFor: 'accessing-defaults' stamp: 'lr 9/27/2006 11:22'!
defaultUsers
	^ PRCurrentContext value kernel users! !

!PUGroup class methodsFor: 'accessing-description' stamp: 'jf 9/17/2009 15:44'!
descriptionUsers
	^ MAMultipleOptionDescription new
		options: [ self defaultUsers ] magritteDynamicObject;
		componentClass: MAListCompositonComponent;
		reference: (PUUser descriptionContainer
				add: PUUser descriptionName;
				yourself);
		propertyAt: #report put: true;
		accessor: #users;
		label: 'Users';
		priority: 200;
		beDistinct;
		yourself! !

!PUGroup class methodsFor: 'testing' stamp: 'lr 3/9/2006 11:31'!
isAbstract
	^ false! !

!PUGroup methodsFor: 'testing' stamp: 'lr 3/9/2006 11:30'!
includes: anUser
	^ self users includes: anUser! !

!PUGroup methodsFor: 'initialization' stamp: 'lr 9/27/2006 12:02'!
initialize
	super initialize.
	users := Set new! !

!PUGroup methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:30'!
users
	^ users! !

!PUGroup methodsFor: 'accessing' stamp: 'lr 2/12/2007 19:07'!
users: aCollection
	(aCollection copyWithoutAll: self users) do: [ :user |
		(user groups includes: self)
			ifFalse: [ user groups add: self ] ].
	(self users copyWithoutAll: aCollection) do: [ :user |
		(user groups includes: self)
			ifTrue: [ user groups remove: self ] ].
	users := aCollection asSet! !

!PUMember class methodsFor: 'accessing-description' stamp: 'lr 4/23/2009 22:38'!
descriptionComment
	^ MAMemoDescription new
		group: 'Additional Information';
		propertyAt: #public put: true;
		accessor: #comment;
		label: 'Comment';
		priority: 990;
		lineCount: 3;
		yourself! !

!PUMember class methodsFor: 'accessing-description' stamp: 'lr 4/23/2009 22:38'!
descriptionName
	^ MAStringDescription new
		propertyAt: #public put: true;
		propertyAt: #report put: true;
		parameterName: 'name';
		accessor: #name;
		label: 'Name';
		priority: 100;
		beReadonly;
		beRequired;
		yourself! !

!PUMember class methodsFor: 'instance creation' stamp: 'lr 3/9/2006 11:31'!
named: aString
	^ self new
		setName: aString;
		yourself! !

!PUMember methodsFor: 'comparing' stamp: 'lr 3/9/2006 11:31'!
= anObject
	^ self class = anObject class
		and: [ self name = anObject name ]! !

!PUMember methodsFor: 'accessing' stamp: 'lr 4/23/2009 22:29'!
comment
	^ comment! !

!PUMember methodsFor: 'accessing' stamp: 'lr 4/23/2009 22:29'!
comment: aString
	comment := aString! !

!PUMember methodsFor: 'comparing' stamp: 'lr 3/9/2006 11:31'!
hash
	^ self class hash bitXor: self name hash! !

!PUMember methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:31'!
name
	^ name! !

!PUMember methodsFor: 'printing' stamp: 'lr 3/9/2006 11:31'!
printOn: aStream
	super printOn: aStream.
	aStream nextPutAll: ' name: '; print: self name! !

!PUMember methodsFor: 'initialization' stamp: 'lr 3/9/2006 11:31'!
setName: aString
	name := aString! !

PUMember subclass: #PUUser
	instanceVariableNames: 'password token groups superuser firstName lastName email'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Security-Model'!

!PUUser class methodsFor: 'accessing-defaults' stamp: 'lr 9/27/2006 10:47'!
defaultGroups
	^ PRCurrentContext value kernel groups! !

!PUUser class methodsFor: 'accessing-description' stamp: 'lr 4/23/2009 22:39'!
descriptionEmail
	^ MAStringDescription new
		group: 'Additional Information';
		propertyAt: #public put: true;
		propertyAt: #report put: true;
		parameterName: 'email';
		accessor: #email;
		label: 'Email';
		priority: 920;
		yourself! !

!PUUser class methodsFor: 'accessing-description' stamp: 'lr 4/23/2009 22:39'!
descriptionFirstName
	^ MAStringDescription new
		group: 'Additional Information';
		propertyAt: #public put: true;
		propertyAt: #report put: true;
		parameterName: 'first-name';
		accessor: #firstName;
		label: 'First Name';
		priority: 900;
		yourself! !

!PUUser class methodsFor: 'accessing-description' stamp: 'lr 5/2/2009 11:44'!
descriptionFullName
	^ MAStringDescription new
		parameterName: 'full-name';
		accessor: #fullName;
		beReadonly;
		beHidden;
		yourself! !

!PUUser class methodsFor: 'accessing-description' stamp: 'jf 9/17/2009 15:44'!
descriptionGroups
	^ MAMultipleOptionDescription new
		addCondition: (MACondition selector: #notEmpty)
			labelled: 'A user has to belong to at least one group';
		options: [ self defaultGroups ] magritteDynamicObject;
		componentClass: MAListCompositonComponent;
		reference: (PUGroup descriptionContainer
			add: PUGroup descriptionName;
			yourself);
		propertyAt: #report put: true;
		accessor: #groups;
		label: 'Groups';
		priority: 300;
		beDistinct;
		beOrdered;
		yourself! !

!PUUser class methodsFor: 'accessing-description' stamp: 'lr 4/23/2009 22:39'!
descriptionLastName
	^ MAStringDescription new
		group: 'Additional Information';
		propertyAt: #public put: true;
		propertyAt: #report put: true;
		parameterName: 'last-name';
		accessor: #lastName;
		label: 'Last Name';
		priority: 910;
		yourself! !

!PUUser class methodsFor: 'accessing-description' stamp: 'lr 4/10/2008 14:17'!
descriptionPassword
	^ MAPasswordDescription new
		accessor: #password;
		label: 'Password';
		priority: 200;
		propertyAt: #public
			put: true;
		beRequired;
		yourself! !

!PUUser class methodsFor: 'accessing-description' stamp: 'lr 4/23/2009 22:39'!
descriptionSuperuser
	^ MABooleanDescription new
		comment: 'Superusers have unrestricted rights, they can also edit users and groups.';
		propertyAt: #report put: true;
		accessor: #superuser;
		label: 'Superuser';
		priority: 400;
		yourself! !

!PUUser class methodsFor: 'testing' stamp: 'lr 3/9/2006 11:31'!
isAbstract
	^ false! !

!PUUser methodsFor: 'utilties' stamp: 'lr 9/27/2006 11:54'!
addGroup: aGroup
	(self groups includes: aGroup)
		ifFalse: [ self groups add: aGroup ].
	aGroup users add: self! !

!PUUser methodsFor: 'accessing' stamp: 'lr 4/23/2009 22:35'!
email
	^ email! !

!PUUser methodsFor: 'accessing' stamp: 'lr 4/23/2009 22:35'!
email: aString
	email := aString! !

!PUUser methodsFor: 'accessing' stamp: 'lr 4/23/2009 22:35'!
firstName
	^ firstName! !

!PUUser methodsFor: 'accessing' stamp: 'lr 4/23/2009 22:35'!
firstName: aString
	firstName := aString! !

!PUUser methodsFor: 'accessing-readonly' stamp: 'lr 5/2/2009 11:41'!
fullName
	"Answer a human readable name of the user."
	
	| full |
	full := String new.
	self firstName notNil
		ifTrue: [ full := full , self firstName , ' ' ].
	self lastName notNil
		ifTrue: [ full := full , self lastName ].
	^ full isEmpty
		ifTrue: [ self name ]
		ifFalse: [ full ]! !

!PUUser methodsFor: 'accessing-readonly' stamp: 'lr 3/9/2006 11:31'!
group
	^ self groups first! !

!PUUser methodsFor: 'accessing-readonly' stamp: 'lr 3/9/2006 11:31'!
groups
	^ groups! !

!PUUser methodsFor: 'accessing-readonly' stamp: 'lr 9/27/2006 12:03'!
groups: aCollection
	self groups do: [ :each | each users remove: self ifAbsent: [ ] ].
	aCollection do: [ :each | each users add: self ].
	groups := aCollection asOrderedCollection! !

!PUUser methodsFor: 'initialization' stamp: 'lr 3/9/2006 11:31'!
initialize
	super initialize.
	groups := OrderedCollection new.
	superuser := false! !

!PUUser methodsFor: 'testing' stamp: 'lr 3/9/2006 11:31'!
isSuperuser
	^ self superuser! !

!PUUser methodsFor: 'accessing' stamp: 'lr 4/23/2009 22:35'!
lastName
	^ lastName! !

!PUUser methodsFor: 'accessing' stamp: 'lr 4/23/2009 22:35'!
lastName: aString
	lastName := aString! !

!PUUser methodsFor: 'tokens' stamp: 'jf 9/14/2009 15:02'!
newToken
	"Creates a new token for cookie authentication."

	^ token := (WAKeyGenerator current keyOfLength: 64) printString! !

!PUUser methodsFor: 'accessing' stamp: 'lr 9/11/2008 16:17'!
password
	^ password isNil ifFalse: [ String new: 8 withAll: $* ]! !

!PUUser methodsFor: 'accessing' stamp: 'jf 11/3/2009 11:07'!
password: aString
	(aString isNil or: [ aString isEmpty or: [ self password = aString ] ])
		ifTrue: [ ^ self ].
	password := GRPlatform current secureHashFor: aString! !

!PUUser methodsFor: 'utilties' stamp: 'lr 3/9/2006 11:31'!
removeGroup: aGroup
	self groups remove: aGroup.
	aGroup users remove: self! !

!PUUser methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:31'!
superuser
	^ superuser! !

!PUUser methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:31'!
superuser: aBoolean
	superuser := aBoolean! !

!PUUser methodsFor: 'accessing-readonly' stamp: 'lr 4/30/2009 15:36'!
token
	"Answer a token used for cookie authentication."

	^ token ifNil: [ self newToken ]! !

!PUUser methodsFor: 'validating' stamp: 'jf 11/3/2009 11:07'!
validatePassword: aString
	^ aString isNil not and: [ aString isEmpty not and: [ (GRPlatform current secureHashFor: aString) = password ] ]! !

PRObject subclass: #PUPermission
	instanceVariableNames: 'command'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Security-Model'!

PUPermission subclass: #PUAddPermission
	instanceVariableNames: 'structure'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Security-Model'!

!PUAddPermission class methodsFor: 'instance creation' stamp: 'jf 10/1/2009 12:09'!
on: aCommand structure: aClass 
	^ (self on: aCommand) setStructure: aClass! !

!PUAddPermission methodsFor: 'comparing' stamp: 'lr 9/11/2006 10:17'!
= anObject
	^ super = anObject
		and: [ self structure = anObject structure ]! !

!PUAddPermission methodsFor: 'comparing' stamp: 'lr 9/11/2006 10:17'!
hash
	^ super hash bitXor: self structure hash! !

!PUAddPermission methodsFor: 'accessing' stamp: 'lr 9/11/2006 10:16'!
label
	^ super label , ' ' , self structure label! !

!PUAddPermission methodsFor: 'initialization' stamp: 'lr 9/11/2006 10:15'!
setStructure: aClass
	structure := aClass! !

!PUAddPermission methodsFor: 'accessing' stamp: 'lr 9/11/2006 10:16'!
structure
	^ structure! !

!PUPermission class methodsFor: 'accessing-description' stamp: 'lr 4/10/2008 14:17'!
descriptionLabel
	^ MAStringDescription new
		accessor: #label;
		yourself! !

!PUPermission class methodsFor: 'testing' stamp: 'lr 9/11/2006 10:10'!
isAbstract
	^ false! !

!PUPermission class methodsFor: 'instance creation' stamp: 'lr 9/11/2006 10:15'!
on: aClass
	^ self new
		setCommand: aClass;
		yourself! !

!PUPermission methodsFor: 'comparing' stamp: 'lr 9/11/2006 10:10'!
= aCommand
	^ self species = aCommand species
		and: [ self command = aCommand command ]! !

!PUPermission methodsFor: 'accessing' stamp: 'lr 9/11/2006 10:09'!
command
	"Answer the command class this permission applies to."

	^ command! !

!PUPermission methodsFor: 'comparing' stamp: 'lr 9/11/2006 10:24'!
hash
	^ self command hash! !

!PUPermission methodsFor: 'accessing' stamp: 'lr 9/11/2006 10:13'!
label
	"Answer the label this command class is refered to in the user interface."

	^ self command label! !

!PUPermission methodsFor: 'printing' stamp: 'lr 9/11/2006 10:10'!
printOn: aStream
	aStream nextPutAll: self label! !

!PUPermission methodsFor: 'initialization' stamp: 'lr 9/11/2006 09:56'!
setCommand: aClass
	command := aClass! !

!PUPermission methodsFor: 'testing' stamp: 'lr 9/11/2006 10:59'!
validCommand: aCommand
	^ self command = aCommand species! !

!PRCommand class methodsFor: '*pier-security' stamp: 'lr 9/11/2006 10:21'!
permissions
	"Answer a collection of permissions as defined by the receiver."

	^ Array with: (PUPermission on: self)! !

!PRCommand methodsFor: '*pier-security' stamp: 'lr 11/6/2007 08:22'!
permission
	"The permission required to make the receiver executable in the setup context. ==nil== if no permission is required."

	| permissions |
	permissions := self class permissions.
	^ permissions isEmpty ifFalse: [ permissions anyOne ]! !

PRCommand subclass: #PUChangeSecurity
	instanceVariableNames: 'with all operator permissions'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Security-Command'!

PUChangeSecurity subclass: #PUChangeGroup
	instanceVariableNames: 'group'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Security-Command'!

!PUChangeGroup class methodsFor: 'accessing' stamp: 'lr 5/3/2006 16:58'!
accessKey
	^ $g! !

!PUChangeGroup class methodsFor: 'accessing-description' stamp: 'jf 9/17/2009 15:43'!
descriptionGroup
	^ MASingleOptionDescription new
		comment: 'Changes the group ownership to the selected group.';
		options: [ self groupInstances ] magritteDynamicObject;
		reference: PUGroup description;
		undefined: '(unchanged)';
		parameterName: 'group';
		accessor: #group;
		label: 'Group';
		priority: 200;
		beSorted;
		yourself! !

!PUChangeGroup class methodsFor: 'testing' stamp: 'lr 3/9/2006 11:31'!
isAbstract
	^ false! !

!PUChangeGroup class methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:31'!
label
	^ 'Change Group'! !

!PUChangeGroup methodsFor: 'configuration' stamp: 'lr 10/6/2008 11:42'!
accessor
	^ #(securityDecoration groupPermissions) asAccessor! !

!PUChangeGroup methodsFor: 'events' stamp: 'lr 3/9/2006 11:30'!
doExecuteFor: aStructure
	self group notNil
		ifTrue: [ aStructure securityDecoration group: self group ].
	super doExecuteFor: aStructure! !

!PUChangeGroup methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:30'!
group
	^ group ifNil: [ group := self structure group ]! !

!PUChangeGroup methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:30'!
group: aGroup
	group := aGroup! !

PUChangeSecurity subclass: #PUChangeOther
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Security-Command'!

!PUChangeOther class methodsFor: 'accessing' stamp: 'lr 5/3/2006 16:58'!
accessKey
	^ $o! !

!PUChangeOther class methodsFor: 'testing' stamp: 'lr 3/9/2006 11:31'!
isAbstract
	^ false! !

!PUChangeOther class methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:31'!
label
	^ 'Change Other'! !

!PUChangeOther methodsFor: 'configuration' stamp: 'lr 10/6/2008 11:42'!
accessor
	^ #(securityDecoration otherPermissions) asAccessor! !

PUChangeSecurity subclass: #PUChangeOwner
	instanceVariableNames: 'owner'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Security-Command'!

!PUChangeOwner class methodsFor: 'accessing-description' stamp: 'jf 9/17/2009 15:43'!
descriptionOwner
	^ MASingleOptionDescription new
		comment: 'Changes the ownership to the selected user.';
		options: [ self userInstances ] magritteDynamicObject;
		reference: PUUser description;
		undefined: '(unchanged)';
		parameterName: 'group';
		accessor: #owner;
		label: 'Owner';
		priority: 200;
		beSorted;
		yourself! !

!PUChangeOwner class methodsFor: 'testing' stamp: 'lr 3/9/2006 11:31'!
isAbstract
	^ false! !

!PUChangeOwner class methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:31'!
label
	^ 'Change Owner'! !

!PUChangeOwner methodsFor: 'accessing' stamp: 'lr 10/6/2008 11:42'!
accessor
	^ #(securityDecoration ownerPermissions) asAccessor! !

!PUChangeOwner methodsFor: 'events' stamp: 'lr 3/9/2006 11:30'!
doExecuteFor: aStructure
	self owner notNil
		ifTrue: [ aStructure securityDecoration owner: self owner ].
	super doExecuteFor: aStructure! !

!PUChangeOwner methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:30'!
owner
	^ owner ifNil: [ owner := self structure owner ]! !

!PUChangeOwner methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:30'!
owner: anUser
	owner := anUser! !

!PUChangeSecurity class methodsFor: 'accessing-defaults' stamp: 'lr 1/12/2007 16:18'!
defaultAll
	^ false! !

!PUChangeSecurity class methodsFor: 'accessing-defaults' stamp: 'lr 3/9/2006 11:31'!
defaultOperator
	^ self defaultOperators first! !

!PUChangeSecurity class methodsFor: 'accessing-defaults' stamp: 'lr 3/9/2006 11:31'!
defaultOperators
	^ #( set add remove )! !

!PUChangeSecurity class methodsFor: 'accessing-defaults' stamp: 'lr 3/9/2006 11:31'!
defaultWith
	^ true! !

!PUChangeSecurity class methodsFor: 'accessing-description' stamp: 'lr 5/31/2009 20:01'!
descriptionAll
	^ MABooleanDescription new
		comment: 'Apply changes recursively to all children.';
		parameterName: 'recursive';
		default: self defaultAll;
		label: 'Recursive';
		accessor: #all;
		priority: 110;
		beRequired;
		yourself! !

!PUChangeSecurity class methodsFor: 'accessing-description' stamp: 'lr 5/31/2009 20:02'!
descriptionOperator
	^ MASingleOptionDescription new
		comment: 'Defines if the permissions should remain unchanged, be set to the selected ones, or be added/removed from the current selection.';
		reference: MASymbolDescription new;
		options: self defaultOperators;
		default: self defaultOperator;
		parameterName: 'operator';
		undefined: '(unchanged)';
		accessor: #operator;
		label: 'Operator';
		priority: 300;
		yourself! !

!PUChangeSecurity class methodsFor: 'accessing-description' stamp: 'jf 9/17/2009 15:43'!
descriptionPermissions
	^ MAMultipleOptionDescription new
		comment: 'Defines the set of permissions used together with the selected operation.';
		options: [ self permissionInstances ] magritteDynamicObject;
		reference: PUPermission description;
		parameterName: 'permissions';
		accessor: #permissions;
		label: 'Permissions';
		priority: 310;
		beSorted;
		yourself! !

!PUChangeSecurity class methodsFor: 'accessing-description' stamp: 'lr 5/31/2009 20:02'!
descriptionWith
	^ MABooleanDescription new
		comment: 'Apply changes to the current structure.';
		parameterName: 'current';
		default: self defaultWith;
		accessor: #with;
		label: 'Current';
		priority: 100;
		beRequired;
		yourself! !

!PUChangeSecurity class methodsFor: 'private' stamp: 'lr 3/9/2006 11:31'!
groupInstances
	^ PRCurrentContext value kernel groups! !

!PUChangeSecurity class methodsFor: 'private' stamp: 'lr 11/5/2007 21:17'!
permissionInstances
	^ PRCommand withAllConcreteClasses gather: [ :each | each permissions ]! !

!PUChangeSecurity class methodsFor: 'private' stamp: 'lr 3/9/2006 11:31'!
userInstances
	^ PRCurrentContext value kernel users! !

!PUChangeSecurity methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:30'!
accessor
	self subclassResponsibility! !

!PUChangeSecurity methodsFor: 'accessing-scope' stamp: 'lr 3/9/2006 11:30'!
all
	^ all ifNil: [ all := self class defaultAll ]! !

!PUChangeSecurity methodsFor: 'accessing-scope' stamp: 'lr 3/9/2006 11:30'!
all: aBoolean
	all := aBoolean! !

!PUChangeSecurity methodsFor: 'events' stamp: 'lr 4/13/2009 11:48'!
doExecute
	| enumerator |
	super doExecute.
	enumerator := self structure enumerator.
	self with ifTrue: [ enumerator with ].
	self all ifTrue: [ enumerator all ].
	enumerator do: [ :each |
		(self all or: [ self structure = each ])
			ifTrue: [ self doExecuteFor: each ] ]! !

!PUChangeSecurity methodsFor: 'events' stamp: 'lr 2/12/2007 19:07'!
doExecuteFor: aStructure
	"Apply the changes to aStructure. Override this method to further modify aStructure. This method is automatically called by #doExecute for all the requested structure."

	self accessor
		write: (self operator = #set
			ifTrue: [ self permissions copy ]
			ifFalse: [
				self operator = #add
					ifTrue: [
						(self accessor read: aStructure)
							copyWithAll: self permissions ]
					ifFalse: [
						self operator = #remove
							ifTrue: [
								(self accessor read: aStructure)
									copyWithoutAll: self permissions ]
							ifFalse: [ ^ self ] ] ])
		to: aStructure! !

!PUChangeSecurity methodsFor: 'accessing-operation' stamp: 'lr 3/9/2006 11:30'!
operator
	^ operator! !

!PUChangeSecurity methodsFor: 'accessing-operation' stamp: 'lr 3/9/2006 11:30'!
operator: aSymbol
	operator := aSymbol! !

!PUChangeSecurity methodsFor: 'accessing-operation' stamp: 'lr 3/9/2006 11:30'!
permissions
	^ permissions ifNil: [ permissions := self accessor read: self structure ]! !

!PUChangeSecurity methodsFor: 'accessing-operation' stamp: 'lr 3/9/2006 11:30'!
permissions: aCollectionOfCommands
	permissions := aCollectionOfCommands asSet! !

!PUChangeSecurity methodsFor: 'accessing' stamp: 'lr 8/11/2007 21:58'!
saveButton
	^ super saveButton value: 'Apply'! !

!PUChangeSecurity methodsFor: 'accessing-scope' stamp: 'lr 3/9/2006 11:30'!
with
	^ with ifNil: [ with := self class defaultWith ]! !

!PUChangeSecurity methodsFor: 'accessing-scope' stamp: 'lr 3/9/2006 11:30'!
with: aBoolean
	with := aBoolean! !

PRCommand subclass: #PUChangeUser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Security-Command'!

!PUChangeUser class methodsFor: 'accessing' stamp: 'lr 11/5/2007 21:15'!
accessKey
	^ $l! !

!PUChangeUser class methodsFor: 'accessing' stamp: 'lr 11/5/2007 21:15'!
permissions
	^ #()! !

!PUChangeUser methodsFor: 'events' stamp: 'lr 2/20/2009 00:33'!
doValidate
	"Change user commands do not require validation."! !

!PUChangeUser methodsFor: 'testing' stamp: 'lr 2/20/2009 00:28'!
isLogged
	^ false! !

PUChangeUser subclass: #PULogin
	instanceVariableNames: 'username password user'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Security-Command'!

!PULogin class methodsFor: 'accessing-description' stamp: 'lr 5/31/2009 20:03'!
descriptionPassword
	^ MAPasswordDescription new
		parameterName: 'password';
		accessor: #password;
		label: 'Password';
		priority: 200;
		beRequired;
		yourself! !

!PULogin class methodsFor: 'accessing-description' stamp: 'lr 5/31/2009 20:03'!
descriptionUsername
	^ MAStringDescription new
		parameterName: 'username';
		accessor: #username;
		label: 'Username';
		priority: 100;
		beRequired;
		yourself! !

!PULogin class methodsFor: 'testing' stamp: 'lr 3/9/2006 11:31'!
isAbstract
	^ false! !

!PULogin class methodsFor: 'testing' stamp: 'lr 3/9/2006 11:31'!
isValidIn: aContext
	^ aContext isLoggedIn not! !

!PULogin class methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:31'!
label
	^ 'Login'! !

!PULogin methodsFor: 'events' stamp: 'lr 10/25/2010 08:49'!
doAnswer
	super doAnswer.
	self successAnswer: (self successAnswer user: self user)! !

!PULogin methodsFor: 'events' stamp: 'lr 2/3/2007 19:23'!
doValidate
	super doValidate.
	self user: (self kernel users
		detect: [ :each |
			each name = self username
				and: [ each validatePassword: self password ] ]
		ifNone: [ self invalidLogin ])! !

!PULogin methodsFor: 'private' stamp: 'lr 2/3/2007 19:25'!
invalidLogin
	PUInvalidLogin
		description: self description
		signal: 'Invalid username or password.'! !

!PULogin methodsFor: 'configuration' stamp: 'lr 2/3/2007 19:25'!
mementoClass
	^ MACachedMemento! !

!PULogin methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:30'!
password
	^ password ifNil: [ password := String new ]! !

!PULogin methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:30'!
password: aString
	password := aString! !

!PULogin methodsFor: 'configuration' stamp: 'lr 8/11/2007 21:58'!
saveButton
	^ super saveButton value: self class label! !

!PULogin methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:30'!
user
	^ user! !

!PULogin methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:30'!
user: aUser
	user := aUser! !

!PULogin methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:30'!
username
	^ username ifNil: [ username := String new ]! !

!PULogin methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:30'!
username: aString
	username := aString! !

PUChangeUser subclass: #PULogout
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Security-Command'!

!PULogout class methodsFor: 'testing' stamp: 'lr 3/9/2006 11:31'!
isAbstract
	^ false! !

!PULogout class methodsFor: 'testing' stamp: 'lr 3/9/2006 11:31'!
isValidIn: aContext
	^ aContext isLoggedIn! !

!PULogout class methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:31'!
label
	^ 'Logout'! !

!PULogout methodsFor: 'events' stamp: 'lr 10/25/2010 08:48'!
doAnswer
	super doAnswer.
	self successAnswer: (self successAnswer user: nil)! !

!PULogout methodsFor: 'events' stamp: 'lr 4/28/2009 22:38'!
doExecute
	super doExecute.
	self context user isNil 
		ifFalse: [ self context user newToken ]! !

!PULogout methodsFor: 'testing' stamp: 'lr 3/9/2006 11:31'!
isQuick
	^ true! !

!PRForbiddenView methodsFor: '*pier-security-rendering' stamp: 'lr 10/25/2010 08:55'!
renderOptionLoginOn: html
	<option: 0>
	
	| loginContext |
	loginContext := self context
		structure: self context structure
		command: (PULogin new
			successAnswer: forbidden;
			yourself).
	loginContext isValid ifTrue: [
		html listItem: [
			html anchor
				goto: loginContext;
				with: 'Authenticate'.
			html text: ' yourself to the site.' ] ]! !
